# This Python file uses the following encoding: utf-8
import os
from pathlib import Path
import sys

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QApplication, QMainWindow
from PyQt5.QtGui import QImage, QPixmap
from form import Ui_MainWindow
from capturethread import CaptureThread

import ctypes
from ctypes import *
from enum import Enum

import warnings
warnings.filterwarnings("ignore", category=DeprecationWarning)

class CONTROL_ID(Enum):
    CONTROL_BRIGHTNESS = 0
    CONTROL_CONTRAST = 1
    CONTROL_WBR = 2
    CONTROL_WBB = 3
    CONTROL_WBG = 4
    CONTROL_GAMMA = 5
    CONTROL_GAIN = 6
    CONTROL_OFFSET = 7
    CONTROL_EXPOSURE = 8
    CONTROL_SPEED = 9
    CONTROL_TRANSFERBIT = 10
    CONTROL_CHANNELS = 11
    CONTROL_USBTRAFFIC = 12
    CONTROL_CURTEMP = 14
    CONTROL_CURPWM = 15
    CONTROL_MANULPWM = 16
    CONTROL_CFWPORT = 17
    CONTROL_COOLER = 18
    CONTROL_ST4PORT = 19
    CAM_COLOR = 20
    CAM_BIN1X1MODE = 21
    CAM_BIN2X2MODE = 22
    CAM_BIN3X3MODE = 23
    CAM_BIN4X4MODE = 24
    CAM_8BITS = 34
    CAM_16BITS = 35
    CAM_GPS = 36
    CONTROL_AMPV = 41
    CONTROL_CFWSLOTSNUM = 44
    CAM_SINGLEFRAMEMODE = 57
    CAM_LIVEVIDEOMODE = 58
    CAM_IS_COLOR = 59

qhyccddll = cdll.LoadLibrary('.\\qhyccd.dll')

# get camera id
qhyccddll.GetQHYCCDId.argtypes = [ctypes.c_uint32, ctypes.c_char_p]
# get handle via camera id
qhyccddll.OpenQHYCCD.argtypes = [ctypes.c_char_p]
qhyccddll.OpenQHYCCD.restype = ctypes.c_void_p
# close camera
qhyccddll.CloseQHYCCD.argtypes = [ctypes.c_void_p]
# read mode
qhyccddll.GetQHYCCDNumberOfReadModes.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_uint32)]
qhyccddll.GetQHYCCDReadModeName.argtypes = [ctypes.c_void_p, ctypes.c_uint32, ctypes.c_char_p]
qhyccddll.GetQHYCCDReadModeResolution.argtypes = [ctypes.c_void_p, ctypes.c_uint32, ctypes.POINTER(ctypes.c_uint32),
                                                 ctypes.POINTER(ctypes.c_uint32)]
qhyccddll.SetQHYCCDReadMode.argtypes = [ctypes.c_void_p, ctypes.c_uint32]
# set single stream mode or live stream mode
qhyccddll.SetQHYCCDStreamMode.argtypes = [ctypes.c_void_p, ctypes.c_uint32]
# initialize camera
qhyccddll.InitQHYCCD.argtypes = [ctypes.c_void_p]
# check available functions
qhyccddll.IsQHYCCDControlAvailable.argtypes = [ctypes.c_void_p, ctypes.c_uint32]
# get camera chip information
qhyccddll.GetQHYCCDChipInfo.argtypes = [ctypes.c_void_p,
                                       ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double),
                                       ctypes.POINTER(ctypes.c_uint32), ctypes.POINTER(ctypes.c_uint32),
                                       ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double),
                                       ctypes.POINTER(ctypes.c_uint32)]
# get parameters min max step
qhyccddll.GetQHYCCDParamMinMaxStep.argtypes = [ctypes.c_void_p, ctypes.c_uint32, ctypes.POINTER(ctypes.c_double),
                                              ctypes.POINTER(ctypes.c_double), ctypes.POINTER(ctypes.c_double)]
# get parameters value
qhyccddll.GetQHYCCDParam.argtypes = [ctypes.c_void_p, ctypes.c_uint32]
qhyccddll.GetQHYCCDParam.restype = ctypes.c_double
# get memory length
qhyccddll.GetQHYCCDMemLength.argtypes = [ctypes.c_void_p]
# set parameters
qhyccddll.SetQHYCCDParam.argtypes = [ctypes.c_void_p, ctypes.c_uint32, ctypes.c_double]
# set debayer on or off, only for color camera
qhyccddll.SetQHYCCDDebayerOnOff.argtypes = [ctypes.c_void_p, ctypes.c_bool]
# set bin mode
qhyccddll.SetQHYCCDBinMode.argtypes = [ctypes.c_void_p, ctypes.c_uint32]
# set resolution and ROI
qhyccddll.SetQHYCCDResolution.argtypes = [ctypes.c_void_p, ctypes.c_uint32, ctypes.c_uint32, ctypes.c_uint32,
                                         ctypes.c_uint32]
# start single stream mode exposing
qhyccddll.ExpQHYCCDSingleFrame.argtypes = [ctypes.c_void_p]
# get single frame data
qhyccddll.GetQHYCCDSingleFrame.argtypes = [ctypes.c_void_p,
                                          ctypes.POINTER(ctypes.c_uint32), ctypes.POINTER(ctypes.c_uint32),
                                          ctypes.POINTER(ctypes.c_uint32), ctypes.POINTER(ctypes.c_uint32),
                                          ctypes.POINTER(ctypes.c_uint8)]
# cancel single exposing and camera will NOT output frame data
qhyccddll.CancelQHYCCDExposingAndReadout.argtypes = [ctypes.c_void_p]
# start live stream mode
qhyccddll.BeginQHYCCDLive.argtypes = [ctypes.c_void_p]
# get live frame data
qhyccddll.GetQHYCCDLiveFrame.argtypes = [ctypes.c_void_p,
                                        ctypes.POINTER(ctypes.c_uint32), ctypes.POINTER(ctypes.c_uint32),
                                        ctypes.POINTER(ctypes.c_uint32), ctypes.POINTER(ctypes.c_uint32),
                                        ctypes.POINTER(ctypes.c_uint8)]
# stop live stream mode
qhyccddll.StopQHYCCDLive.argtypes = [ctypes.c_void_p]
# convert image data
qhyccddll.Bits16ToBits8.argtypes = [ctypes.c_void_p,
                                      ctypes.POINTER(ctypes.c_uint8), ctypes.POINTER(ctypes.c_uint8),
                                      ctypes.c_uint32, ctypes.c_uint32, ctypes.c_uint16, ctypes.c_uint16]

camhandle = 0 # camera handle
connected = False # connect flag
imgdata = 0
frameCount = 0

class MainWindow(QMainWindow, Ui_MainWindow):
#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def __init__(self):
        super(MainWindow, self).__init__()
        self.setupUi(self)
        self.comboBox_ExpUnit.setCurrentText("ms")
        self.destroyed.connect(lambda: self.CleanupFun(self))
        self.pushButton_Scan.clicked.connect(lambda: self.ScanFun(self))
        self.comboBox_CamList.currentTextChanged.connect(lambda: self.On_comboBox_CamList_currentTextChanged(self))
        self.pushButton_Connect.clicked.connect(lambda: self.ConnectFun(self))
        self.pushButton_Disconnect.clicked.connect(lambda: self.DisconnectFun(self))
        self.pushButton_Capture.clicked.connect(lambda: self.CaptureFun(self))
        self.pushButton_Cancel.clicked.connect(lambda: self.CancelFun(self))
        self.comboBox_ReadMode.currentTextChanged.connect(lambda: self.On_comboBox_ReadMode_currentTextChanged(self))
        self.comboBox_Bin.currentTextChanged.connect(lambda: self.On_comboBox_Bin_currentTextChanged(self))
        self.comboBox_ExpUnit.currentTextChanged.connect(lambda: self.On_comboBox_ExpUnit_currentTextChanged(self))
        self.hSlider_ExpTime.valueChanged.connect(lambda: self.On_hSlider_ExpTime_valueChanged(self))
        self.spinBox_ExpTime.valueChanged.connect(lambda: self.On_spinBox_ExpTime_valueChanged(self))
        self.hSlider_Gain.valueChanged.connect(lambda: self.On_hSlider_Gain_valueChanged(self))
        self.spinBox_Gain.valueChanged.connect(lambda: self.On_spinBox_Gain_valueChanged(self))
        self.hSlider_Offset.valueChanged.connect(lambda: self.On_hSlider_Offset_valueChanged(self))
        self.spinBox_Offset.valueChanged.connect(lambda: self.On_spinBox_Offset_valueChanged(self))

        # call firstly, just need once
        ret = qhyccddll.InitQHYCCDResource()
        print("__init__() InitQHYCCDResource() ret =", ret)
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    # call finally, just need once
    def CleanupFun(address, self):
        ret = qhyccddll.ReleaseQHYCCDResource()
        print("CleanupFun() ReleaseQHYCCDResource() ret =", ret)
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def ScanFun(address, self):
        num = qhyccddll.ScanQHYCCD();
        print("ScanFun() ScanQHYCCD() num =", num)
        if num <= 0:
            return -1
        self.textBrowser_Output.append("Find " + str(num) + " camera ...")

        self.comboBox_CamList.blockSignals(True)
        self.comboBox_CamList.clear()
        for i in range(num):
            id = ctypes.create_string_buffer(40)
            ret = qhyccddll.GetQHYCCDId(i, id)
            print("ScanFun() GetQHYCCDId() ret =", ret, "id =", id.value.decode("utf-8"))
            if ret == 0:
                self.comboBox_CamList.addItem(id.value.decode("utf-8"))
                camid = id
        self.comboBox_CamList.blockSignals(False)

        global camhandle
        id = ctypes.c_char_p(self.comboBox_CamList.currentText().encode("utf-8"))
        camhandle = qhyccddll.OpenQHYCCD(id)
        print("ScanFun() OpenQHYCCD() camhandle =", hex(camhandle))
        if camhandle == 0:
            return -1

        self.comboBox_ReadMode.blockSignals(True)
        self.comboBox_ReadMode.clear()
        num = ctypes.c_uint32()
        ret = qhyccddll.GetQHYCCDNumberOfReadModes(camhandle, byref(num))
        print("ScanFun() GetQHYCCDNumberOfReadModes() ret =", ret, "num =", num)
        for i in range(num.value):
            name = ctypes.create_string_buffer(40)
            ret = qhyccddll.GetQHYCCDReadModeName(camhandle, i, name)
            print("ScanFun() GetQHYCCDReadModeName() ret =", ret, "index =", i, "name =", name.value.decode("utf-8"))
            if ret == 0:
                self.comboBox_ReadMode.addItem(name.value.decode("utf-8"))
        self.comboBox_ReadMode.blockSignals(False)
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def On_comboBox_CamList_currentTextChanged(address, self):
        if self.comboBox_CamList.currentText() == "":
            return -1

        global camhandle
        id = ctypes.c_char_p(self.comboBox_CamList.currentText().encode("utf-8"))
        camhandle = qhyccddll.OpenQHYCCD(id)
        print("On_comboBox_CamList_currentTextChanged() OpenQHYCCD() camhandle =", hex(camhandle))
        if camhandle == 0:
            return -1

        self.comboBox_ReadMode.blockSignals(True)
        self.comboBox_ReadMode.clear()
        num = ctypes.c_uint32()
        ret = qhyccddll.GetQHYCCDNumberOfReadModes(camhandle, byref(num))
        print("On_comboBox_CamList_currentTextChanged() GetQHYCCDNumberOfReadModes() ret =", ret, "num =", num)
        for i in range(num.value):
            name = ctypes.create_string_buffer(40)
            ret = qhyccddll.GetQHYCCDReadModeName(camhandle, i, name)
            print("On_comboBox_CamList_currentTextChanged() GetQHYCCDReadModeName() ret =", ret, "index =", i, "num =", num)
            if ret == 0:
                self.comboBox_ReadMode.addItem(name.value.decode("utf-8"))
        self.comboBox_ReadMode.blockSignals(False)
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def ConnectFun(address, self):
        self.textBrowser_Output.append("Connect " + self.comboBox_CamList.currentText() + ", start to initialize ...")
        QApplication.processEvents();

        global camhandle
        readmode = self.comboBox_ReadMode.currentIndex()
        ret = qhyccddll.SetQHYCCDReadMode(camhandle, readmode)
        print("ConnectFun() SetQHYCCDReadMode() readmode =", readmode, "ret =", ret)

        ret = qhyccddll.SetQHYCCDStreamMode(camhandle, 0)
        print("ConnectFun() SetQHYCCDStreamMode() single mode ret =", ret)

        ret = qhyccddll.InitQHYCCD(camhandle)
        print("ConnectFun() InitQHYCCD() ret =", ret)
        self.textBrowser_Output.append("Initialize finished ...")

        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_TRANSFERBIT.value, 16.0)
        print("ConnectFun() SetQHYCCDParam() 16 bits ret =", ret)

        ret = qhyccddll.SetQHYCCDDebayerOnOff(camhandle, False)
        print("ConnectFun() SetQHYCCDDebayerOnOff() False ret =", ret)

        chipW = ctypes.c_double()
        chipH = ctypes.c_double()
        imageW = ctypes.c_uint32()
        imageH = ctypes.c_uint32()
        pixelW = ctypes.c_double()
        pixelH = ctypes.c_double()
        imageB = ctypes.c_uint32()
        ret = qhyccddll.GetQHYCCDChipInfo(camhandle, byref(chipW), byref(chipH), byref(imageW), byref(imageH), byref(pixelW), byref(pixelH), byref(imageB))
        print("ConnectFun() GetQHYCCDChipInfo() ret =", ret)
        print("ConnectFun() GetQHYCCDChipInfo() chip size =", chipW.value, "x", chipH.value, "mm")
        print("ConnectFun() GetQHYCCDChipInfo() pixel size =", pixelW.value, "x", chipH.value, "um")
        print("ConnectFun() GetQHYCCDChipInfo() image size =", imageW.value, "x", imageH.value)

        self.comboBox_Bin.blockSignals(True)
        self.comboBox_Bin.clear()
        ret = qhyccddll.IsQHYCCDControlAvailable(camhandle, CONTROL_ID.CAM_BIN1X1MODE.value)
        print("ConnectFun() IsQHYCCDControlAvailable() CAM_BIN1X1MODE ret =", ret)
        if ret == 0: self.comboBox_Bin.addItem("1x1")
        ret = qhyccddll.IsQHYCCDControlAvailable(camhandle, CONTROL_ID.CAM_BIN2X2MODE.value)
        print("ConnectFun() IsQHYCCDControlAvailable() CAM_BIN2X2MODE ret =", ret)
        if ret == 0: self.comboBox_Bin.addItem("2x2")
        ret = qhyccddll.IsQHYCCDControlAvailable(camhandle, CONTROL_ID.CAM_BIN3X3MODE.value)
        print("ConnectFun() IsQHYCCDControlAvailable() CAM_BIN3X3MODE ret =", ret)
        if ret == 0: self.comboBox_Bin.addItem("3x3")
        ret = qhyccddll.IsQHYCCDControlAvailable(camhandle, CONTROL_ID.CAM_BIN4X4MODE.value)
        print("ConnectFun() IsQHYCCDControlAvailable() CAM_BIN4X4MODE ret =", ret)
        if ret == 0: self.comboBox_Bin.addItem("4x4")
        self.comboBox_Bin.blockSignals(False)

        ret = qhyccddll.SetQHYCCDBinMode(camhandle, 1, 1)
        print("ConnectFun() SetQHYCCDBinMode() 1x1 ret =", ret)

        ret = qhyccddll.SetQHYCCDResolution(camhandle, 0, 0, imageW.value, imageH.value)
        print("ConnectFun() SetQHYCCDResolution() x = 0", "y = 0", "sizex =", imageW.value, "sizey =", imageH.value, "ret =", ret)

        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_EXPOSURE.value, 20000.0)
        print("ConnectFun() SetQHYCCDParam() 20 ms ret =", ret)

        min = ctypes.c_double()
        max = ctypes.c_double()
        step = ctypes.c_double()
        ret = qhyccddll.GetQHYCCDParamMinMaxStep(camhandle, CONTROL_ID.CONTROL_GAIN.value, byref(min), byref(max), byref(step))
        print("ConnectFun() GetQHYCCDParamMinMaxStep() CONTROL_GAIN ret =", ret, "min =", min.value, "max =", max.value, "step =", step.value)
        self.hSlider_Gain.blockSignals(True)
        self.hSlider_Gain.setMinimum(int(min.value))
        self.hSlider_Gain.setMaximum(int(max.value))
        self.hSlider_Gain.blockSignals(False)
        self.spinBox_Gain.blockSignals(True)
        self.spinBox_Gain.setMinimum(int(min.value))
        self.spinBox_Gain.setMaximum(int(max.value))
        self.spinBox_Gain.blockSignals(False)
        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_GAIN.value, 30.0)
        print("ConnectFun() SetQHYCCDParam() gain 30 ret =", ret)

        ret = qhyccddll.GetQHYCCDParamMinMaxStep(camhandle, CONTROL_ID.CONTROL_OFFSET.value, byref(min), byref(max), byref(step))
        print("ConnectFun() GetQHYCCDParamMinMaxStep() CONTROL_OFFSET ret =", ret, "min =", min.value, "max =", max.value, "step =", step.value)
        self.hSlider_Offset.blockSignals(True)
        self.hSlider_Offset.setMinimum(int(min.value))
        self.hSlider_Offset.setMaximum(int(max.value))
        self.hSlider_Offset.blockSignals(False)
        self.spinBox_Offset.blockSignals(True)
        self.spinBox_Offset.setMinimum(int(min.value))
        self.spinBox_Offset.setMaximum(int(max.value))
        self.spinBox_Offset.blockSignals(False)
        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_OFFSET.value, 50.0)
        print("ConnectFun() SetQHYCCDParam() offset 50 ret =", ret)

        length = qhyccddll.GetQHYCCDMemLength(camhandle)
        global imgdata
        imgdata = (ctypes.c_uint8 * length)()

        global connected
        connected = True
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def DisconnectFun(address, self):
        global camhandle
        ret = qhyccddll.CloseQHYCCD(camhandle)
        global connected
        connected = False
        global frameCount
        frameCount = 0
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def ProcessImageData(self, w, h, b, c):
        length = w * h * 4
        showdata = (ctypes.c_uint8 * length)()
        qhyccddll.Bits16ToBits8(camhandle, imgdata, showdata, w, h, 0, 65535)

        ImageOri = QImage(showdata, w, h, w * c, QImage.Format_Grayscale8)
        ImageShow = ImageOri.scaled(self.label_Show.width(), self.label_Show.height(), Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
        self.label_Show.setPixmap(QPixmap.fromImage(ImageShow))

        global frameCount
        frameCount = frameCount + 1
        self.label_FrameCount.setText("Count : " + str(frameCount))
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def CaptureFun(address, self):
        global camhandle
        global imgdata
        self.captureThread = CaptureThread(dll=qhyccddll, handle = camhandle, data = imgdata)
        self.captureThread.capturedFrame.connect(self.ProcessImageData)
        self.captureThread.start()
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def CancelFun(address, self):
        global camhandle
        ret = qhyccddll.CancelQHYCCDExposingAndReadout(camhandle)
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def On_comboBox_ReadMode_currentTextChanged(address, self):
        global connected
        if not connected:
            return -1

        global camhandle
        readmode = self.comboBox_ReadMode.currentIndex()
        ret = qhyccddll.SetQHYCCDReadMode(camhandle, readmode)
        print("On_comboBox_ReadMode_currentTextChanged() SetQHYCCDReadMode() readmode =", readmode, "ret =", ret)

        ret = qhyccddll.SetQHYCCDStreamMode(camhandle, 0)
        print("On_comboBox_ReadMode_currentTextChanged() SetQHYCCDStreamMode() single mode ret =", ret)

        ret = qhyccddll.InitQHYCCD(camhandle)
        print("On_comboBox_ReadMode_currentTextChanged() InitQHYCCD() ret =", ret)
        self.textBrowser_Output.append("Initialize finished ...")

        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_TRANSFERBIT.value, 16.0)
        print("On_comboBox_ReadMode_currentTextChanged() SetQHYCCDParam() 16 bits ret =", ret)

        ret = qhyccddll.SetQHYCCDDebayerOnOff(camhandle, False)
        print("On_comboBox_ReadMode_currentTextChanged() SetQHYCCDDebayerOnOff() False ret =", ret)

        chipW = ctypes.c_double()
        chipH = ctypes.c_double()
        imageW = ctypes.c_uint32()
        imageH = ctypes.c_uint32()
        pixelW = ctypes.c_double()
        pixelH = ctypes.c_double()
        imageB = ctypes.c_uint32()
        ret = qhyccddll.GetQHYCCDChipInfo(camhandle, byref(chipW), byref(chipH), byref(imageW), byref(imageH), byref(pixelW), byref(pixelH), byref(imageB))
        print("On_comboBox_ReadMode_currentTextChanged() GetQHYCCDChipInfo() ret =", ret)
        print("On_comboBox_ReadMode_currentTextChanged() GetQHYCCDChipInfo() chip size =", chipW.value, "x", chipH.value, "mm")
        print("On_comboBox_ReadMode_currentTextChanged() GetQHYCCDChipInfo() pixel size =", pixelW.value, "x", chipH.value, "um")
        print("On_comboBox_ReadMode_currentTextChanged() GetQHYCCDChipInfo() image size =", imageW.value, "x", imageH.value)

        self.comboBox_Bin.blockSignals(True)
        self.comboBox_Bin.clear()
        ret = qhyccddll.IsQHYCCDControlAvailable(camhandle, CONTROL_ID.CAM_BIN1X1MODE.value)
        print("On_comboBox_ReadMode_currentTextChanged() IsQHYCCDControlAvailable() CAM_BIN1X1MODE ret =", ret)
        if ret == 0: self.comboBox_Bin.addItem("1x1")
        ret = qhyccddll.IsQHYCCDControlAvailable(camhandle, CONTROL_ID.CAM_BIN2X2MODE.value)
        print("On_comboBox_ReadMode_currentTextChanged() IsQHYCCDControlAvailable() CAM_BIN2X2MODE ret =", ret)
        if ret == 0: self.comboBox_Bin.addItem("2x2")
        ret = qhyccddll.IsQHYCCDControlAvailable(camhandle, CONTROL_ID.CAM_BIN3X3MODE.value)
        print("On_comboBox_ReadMode_currentTextChanged() IsQHYCCDControlAvailable() CAM_BIN3X3MODE ret =", ret)
        if ret == 0: self.comboBox_Bin.addItem("3x3")
        ret = qhyccddll.IsQHYCCDControlAvailable(camhandle, CONTROL_ID.CAM_BIN4X4MODE.value)
        print("On_comboBox_ReadMode_currentTextChanged() IsQHYCCDControlAvailable() CAM_BIN4X4MODE ret =", ret)
        if ret == 0: self.comboBox_Bin.addItem("4x4")
        self.comboBox_Bin.blockSignals(False)

        ret = qhyccddll.SetQHYCCDBinMode(camhandle, 1, 1)
        print("On_comboBox_ReadMode_currentTextChanged() SetQHYCCDBinMode() 1x1 ret =", ret)

        ret = qhyccddll.SetQHYCCDResolution(camhandle, 0, 0, imageW.value, imageH.value)
        print("On_comboBox_ReadMode_currentTextChanged() SetQHYCCDResolution() x = 0", "y = 0", "sizex =", imageW.value, "sizey =", imageH.value, "ret =", ret)

        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_EXPOSURE.value, 20000.0)
        print("On_comboBox_ReadMode_currentTextChanged() SetQHYCCDParam() 20 ms ret =", ret)

        min = ctypes.c_double()
        max = ctypes.c_double()
        step = ctypes.c_double()
        ret = qhyccddll.GetQHYCCDParamMinMaxStep(camhandle, CONTROL_ID.CONTROL_GAIN.value, byref(min), byref(max), byref(step))
        print("On_comboBox_ReadMode_currentTextChanged() GetQHYCCDParamMinMaxStep() CONTROL_GAIN ret =", ret, "min =", min.value, "max =", max.value, "step =", step.value)
        self.hSlider_Gain.blockSignals(True)
        self.hSlider_Gain.setMinimum(int(min.value))
        self.hSlider_Gain.setMaximum(int(max.value))
        self.hSlider_Gain.blockSignals(False)
        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_GAIN.value, 30.0)
        print("On_comboBox_ReadMode_currentTextChanged() SetQHYCCDParam() gain 30 ret =", ret)

        ret = qhyccddll.GetQHYCCDParamMinMaxStep(camhandle, CONTROL_ID.CONTROL_OFFSET.value, byref(min), byref(max), byref(step))
        print("On_comboBox_ReadMode_currentTextChanged() GetQHYCCDParamMinMaxStep() CONTROL_OFFSET ret =", ret, "min =", min.value, "max =", max.value, "step =", step.value)
        self.hSlider_Offset.blockSignals(True)
        self.hSlider_Offset.setMinimum(int(min.value))
        self.hSlider_Offset.setMaximum(int(max.value))
        self.hSlider_Offset.blockSignals(False)
        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_OFFSET.value, 50.0)
        print("On_comboBox_ReadMode_currentTextChanged() SetQHYCCDParam() offset 50 ret =", ret)

        length = qhyccddll.GetQHYCCDMemLength(camhandle)
        global imgdata
        imgdata = (ctypes.c_uint8 * length)()
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def On_comboBox_Bin_currentTextChanged(address, self):
        global camhandle
        chipW = ctypes.c_double()
        chipH = ctypes.c_double()
        imageW = ctypes.c_uint32()
        imageH = ctypes.c_uint32()
        pixelW = ctypes.c_double()
        pixelH = ctypes.c_double()
        imageB = ctypes.c_uint32()
        ret = qhyccddll.GetQHYCCDChipInfo(camhandle, byref(chipW), byref(chipH), byref(imageW), byref(imageH), byref(pixelW), byref(pixelH), byref(imageB))
        print("On_comboBox_ReadMode_currentTextChanged() GetQHYCCDChipInfo() ret =", ret)
        print("On_comboBox_ReadMode_currentTextChanged() GetQHYCCDChipInfo() chip size =", chipW.value, "x", chipH.value, "mm")
        print("On_comboBox_ReadMode_currentTextChanged() GetQHYCCDChipInfo() pixel size =", pixelW.value, "x", chipH.value, "um")
        print("On_comboBox_ReadMode_currentTextChanged() GetQHYCCDChipInfo() image size =", imageW.value, "x", imageH.value)
        print("On_comboBox_ReadMode_currentTextChanged() comboBox_Bin currentText =", self.comboBox_Bin.currentText())
        if self.comboBox_Bin.currentText() == "1x1":
            ret = qhyccddll.SetQHYCCDBinMode(camhandle, 1, 1)
            ret = qhyccddll.SetQHYCCDResolution(camhandle, 0, 0, imageW.value, imageH.value)
        if self.comboBox_Bin.currentText() == "2x2":
            ret = qhyccddll.SetQHYCCDBinMode(camhandle, 2, 2)
            ret = qhyccddll.SetQHYCCDResolution(camhandle, 0, 0, int(imageW.value / 2), int(imageH.value / 2))
        if self.comboBox_Bin.currentText() == "3x3":
            ret = qhyccddll.SetQHYCCDBinMode(camhandle, 3, 3)
            ret = qhyccddll.SetQHYCCDResolution(camhandle, 0, 0, int(imageW.value / 3), int(imageH.value / 3))
        if self.comboBox_Bin.currentText() == "4x4":
            ret = qhyccddll.SetQHYCCDBinMode(camhandle, 4, 4)
            ret = qhyccddll.SetQHYCCDResolution(camhandle, 0, 0, int(imageW.value / 4), int(imageH.value / 4))
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def On_comboBox_ExpUnit_currentTextChanged(address, self):
        global camhandle
        exptime = self.hSlider_ExpTime.value()
        if self.comboBox_ExpUnit.currentText() == "us":
            exptime = exptime * 1.0
        if self.comboBox_ExpUnit.currentText() == "ms":
            exptime = exptime * 1000.0
        if self.comboBox_ExpUnit.currentText() == "s":
            exptime = exptime * 1000000.0
        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_EXPOSURE.value, exptime)
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def On_hSlider_ExpTime_valueChanged(address, self):
        global camhandle
        exptime = self.hSlider_ExpTime.value()
        if self.comboBox_ExpUnit.currentText() == "us":
            exptime = exptime * 1.0
        if self.comboBox_ExpUnit.currentText() == "ms":
            exptime = exptime * 1000.0
        if self.comboBox_ExpUnit.currentText() == "s":
            exptime = exptime * 1000000.0
        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_EXPOSURE.value, exptime)
        self.spinBox_ExpTime.blockSignals(True)
        self.spinBox_ExpTime.setValue(self.hSlider_ExpTime.value())
        self.spinBox_ExpTime.blockSignals(False)
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def On_spinBox_ExpTime_valueChanged(address, self):
        global camhandle
        exptime = self.spinBox_ExpTime.value()
        if self.comboBox_ExpUnit.currentText() == "us":
            exptime = exptime * 1.0
        if self.comboBox_ExpUnit.currentText() == "ms":
            exptime = exptime * 1000.0
        if self.comboBox_ExpUnit.currentText() == "s":
            exptime = exptime * 1000000.0
        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_EXPOSURE.value, exptime)
        self.hSlider_ExpTime.blockSignals(True)
        self.hSlider_ExpTime.setValue(self.spinBox_ExpTime.value())
        self.hSlider_ExpTime.blockSignals(False)
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def On_hSlider_Gain_valueChanged(address, self):
        global camhandle
        gain = self.hSlider_Gain.value()
        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_GAIN.value, gain)
        self.spinBox_Gain.blockSignals(True)
        self.spinBox_Gain.setValue(self.hSlider_Gain.value())
        self.spinBox_Gain.blockSignals(False)
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def On_spinBox_Gain_valueChanged(address, self):
        global camhandle
        gain = self.spinBox_Gain.value()
        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_GAIN.value, gain)
        self.hSlider_Gain.blockSignals(True)
        self.hSlider_Gain.setValue(self.spinBox_Gain.value())
        self.hSlider_Gain.blockSignals(False)
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def On_hSlider_Offset_valueChanged(address, self):
        global camhandle
        offset = self.hSlider_Offset.value()
        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_OFFSET.value, offset)
        self.spinBox_Offset.blockSignals(True)
        self.spinBox_Offset.setValue(self.hSlider_Offset.value())
        self.spinBox_Offset.blockSignals(False)
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    def On_spinBox_Offset_valueChanged(address, self):
        global camhandle
        offset = self.spinBox_Offset.value()
        ret = qhyccddll.SetQHYCCDParam(camhandle, CONTROL_ID.CONTROL_OFFSET.value, offset)
        self.hSlider_Offset.blockSignals(True)
        self.hSlider_Offset.setValue(self.spinBox_Offset.value())
        self.hSlider_Offset.blockSignals(False)
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

#    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
if __name__ == "__main__":
    app = QApplication([])
    widget = MainWindow()
    widget.show()
    sys.exit(app.exec_())
#    ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑
